self.SetHudHideFlags(/*HIDEHUD_PIPES_AND_CHARGE | HIDEHUD_METAL |*/ HIDEHUD_BUILDING_STATUS | HIDEHUD_CLOAK_AND_FEIGN);

if (redTeamOnly && self.GetTeam() != TF_TEAM_RED)
{
    self.SwitchTeam(TF_TEAM_RED);
    self.ForceRespawn();
}

local mercModel = null;
local playerInTaunt = false;

AddTimer(-1, -10, function()
{
    local inCond = self.InCond(TF_COND_TAUNTING);
    if (inCond && !playerInTaunt)
    {
        playerInTaunt = true;
        KillIfValid(mercModel);
    }
    else if (!inCond && playerInTaunt)
    {
        playerInTaunt = false;
    }
});

OnSelfEvent("init_custom_character", 0, function()
{
    if (!IsValidRound())
        return;
    CreateViewModel(characterSettings.viewmodel);
    //mercModel = CreateWorldModel("models/dmgg/player/merc_model.mdl");

    SetPropInt(self, "m_Shared.m_bShieldEquipped", 0);
    AssembleArsenal();

    //Add Merc's attributes
    self.AddCustomAttribute("cancel falling damage", 1, -1);
    self.AddCustomAttribute("crit mod disabled hidden", 0, -1);
    self.AddCustomAttribute("voice pitch scale", 0, -1);
    self.AddCustomAttribute("no double jump", 1, -1);
    self.AddCond(TF_COND_BURNING_PYRO);

    //Steam-bound clothing color
    SetEntityColor(CalcMercColor());

    //On-spawn invul
    SetPropInt(self, "m_bForcedSkin", 1);
    SetPropInt(self, "m_nForcedSkin", 5);
    if (GetRoundState() == 3)
        self.AddCustomAttribute("no_attack", 1, 5.2);
    self.AddCondEx(TF_COND_INVULNERABLE, GetRoundState() == 3 ? 5 : 2, null);
    RunWithDelay(GetRoundState() == 3 ? 5 : 2, function() {
        SetPropInt(self, "m_nForcedSkin", 4);
    });

    //Spawn sound
    if (GetRoundState() >= 4)
        BuildSound("respawn_sfx").Play();
});

function RefreshCharacter(keepCurrentHP = true)
{
    local oldHP = self.GetHealth();
    local weaponTT = GetWeaponTypeFromEntity(self.GetActiveWeapon());
    local playerClass = safeget(weaponTT, "playerclass", null);

    if (playerClass)
        self.SetPlayerClass(playerClass);
    else
        playerClass = self.GetPlayerClass();
    self.AddCustomAttribute("move speed bonus",
        1.04 / MERC_MOVESPEED_MULTIPLIER[playerClass],
        -1);

    FixViewmodels();

    local model = safeget(weaponTT, "playermodel", null);
    if (model)
        self.SetCustomModelWithClassAnimations(model);

    self.SetHealth(keepCurrentHP ? oldHP : 150);
    self.SetMaxHealth(150);
    self.RemoveCustomAttribute("max health additive penalty");
    self.AddCustomAttribute("max health additive penalty", MERC_HP_OFFSETS[playerClass], -1);
}

OnSelfEvent("player_death", -100, function()
{
    self.SetScriptOverlayMaterial("");
    EntFireByHandle(game_text_hop_bar, "AddOutput", "message ", 0, self, self);
    EntFireByHandle(game_text_hop_bar, "Display", "", 0, self, self);
    BuildSound("death").Voice().SoundLevel(90).Play();
});

OnSelfEvent("player_death_next", 0, function()
{
    local ragdoll = GetPropEntity(self, "m_hRagdoll");
    SetPropBool(ragdoll, "m_bGib", false);
    SetPropInt(ragdoll, "m_clrRender", 4444444);
});

OnSelfEvent("player_death_post", 0, function()
{
    local ragdoll = GetPropEntity(self, "m_hRagdoll");
    SetPropBool(ragdoll, "m_bGib", false);
    SetPropInt(ragdoll, "m_clrRender", 4444444);
});

OnSelfEvent("OnDealDamage", function(params) {
    if ((params.damage_type & 128) && self.InCond(TF_COND_BLASTJUMPING))
        params.damage *= 2;
});

local moveKeys = IN_ATTACK | IN_ATTACK2 | IN_FORWARD | IN_BACK | IN_MOVELEFT | IN_MOVERIGHT;

AddTimer(-1, 0, function()
{
    if (GetPropInt(self, "m_nButtons") & moveKeys && GetRoundState() > 3)
    {
        self.RemoveCond(TF_COND_INVULNERABLE);
        SetPropInt(self, "m_nForcedSkin", 4);
    }
})

function SetEntityColor(rgb)
{
    local color = rgb[0] | (rgb[1] << 8) | (rgb[2] << 16);
    SetPropInt(self, "m_clrRender", color);
}

function CalcMercColor()
{
    local steamId = GetPropString(self, "m_szNetworkIDString");
    local indexStr = steamId.len() > 5 ? steamId.slice(5) : "4444444";
    local index = indexStr.tointeger();
    return [index % 256, (index >> 8) % 256, (index >> 16) % 256, 255];
}